from collections import OrderedDict

from base import Dictifier, ErrorAggregator, Invalid, Field

class Mapping(Dictifier):
    '''Dictifier for structured dicts.

    The field_types variable describes the type to dictify as an ordered
    dictionary mapping dictionary keys to dictifiers for those keys.

    >>> from basic import Int
    >>> from datetypes import Date
    >>> from datetime import date
    >>> bday = date(1985, 9, 16)
    >>> schema = Mapping(dict(age=Int(), birthday=Date()))
    >>> cstruct = schema.dictify(dict(age=27, birthday=bday))
    >>> cstruct == dict(age=27, birthday=u'09/16/1985')
    True
    >>> schema.undictify(cstruct) == dict(age=27, birthday=bday)
    True
    >>> schema.validate(dict(age=27, birthday=bday))
    >>> schema.validate(dict(age=27.1, birthday="not a date"))
    Traceback (most recent call last):
    ...
    Invalid: age: [type_error], birthday: [type_error]

    The 'extra_field_policy' flag should be 'save', 'ignore', or 'error'. This
    controls what happens on undictifiance when unexpected fields are present.

    On 'save', extra fields are stored in the returned dictionary under the key
    _extras:

    >>> schema = Mapping(dict(age=Int()), 'save')
    >>> v = schema.undictify({'age':27, 'color':'blue'})
    >>> v == {'age': 27, '_extras': {'color': 'blue'}}
    True

    On 'error', extra fields cause an Invalid exception:

    >>> schema = Mapping(dict(age=Int()), 'error')
    >>> v = schema.undictify({'age':27, 'color':'blue'})
    Traceback (most recent call last):
    ...
    Invalid: ('unexpected_fields', {'keys': set(['color'])})

    On 'ignore', extra fields are discarded:

    >>> schema = Mapping(dict(age=Int()), 'ignore')
    >>> v = schema.undictify({'age':27, 'color':'blue'})
    >>> v == {'age': 27}
    True

    If the entries in field_types are Fields, they are used directly; if not,
    they're wrapped in Fields with not_empty=False (so that None is legal):

    >>> schema = Mapping(dict(age=Int()))
    >>> schema.validate({'age':None})
    >>> schema = Mapping(dict(age=Field(Int(), not_empty=True)))
    >>> schema.validate({'age':None})
    Traceback (most recent call last):
    ...
    Invalid: age: [empty]
    '''
    def __init__(self, field_types, extra_field_policy="ignore"):
        super(Mapping, self).__init__()
        self.field_types = OrderedDict()
        for key, field_or_dfier in field_types.iteritems():
            self.field_types[key] = Field.asfield(field_or_dfier)
        self.extra_field_policy=extra_field_policy

    def _handle_fields(self, value, error_agg, kwargs):
        '''Undictify our fields. Override this to customize subfields.'''
        data = {}
        for key, type in self.field_types.iteritems():
            with error_agg.checking_sub(key):
                val = value.get(key, None)
                data[key] = type.undictify(val, **kwargs)
        return data

    def _handle_extras(self, value, error_agg, kwargs):
        '''Process any keys in value that aren't expected fields.

        If self.extra_field_policy is 'save', these values will be stored in
        _extras; if it's 'error', they'll be added as errors. Otherwise they'll
        be ignored.
        '''
        if self.extra_field_policy not in ['save', 'error']:
            return None
        extra_keys = set(value.keys()) - set(self.field_types.keys())
        if not extra_keys: return
        if self.extra_field_policy == 'save':
            extras = dict()
            for key in extra_keys:
                extras[key] = value[key]
            return extras
        else:
            error_agg.own_error(Invalid('unexpected_fields', keys=extra_keys))
            return None

    def undictify(self, value, **kwargs):
        if value is None:
            value = {}
        if not isinstance(value, dict):
            raise Invalid('not_dict')
        # If fail_early is True, then gather all errors from this and its
        # children. Otherwise, just raise the first error we encounter.
        error_agg = ErrorAggregator(autoraise = kwargs.get('fail_early', False))
        data = self._handle_fields(value, error_agg, kwargs)
        extras = self._handle_extras(value, error_agg, kwargs)
        if extras:
            data['_extras'] = extras
        error_agg.raise_if_any()
        return data

    def dictify(self, value, **kwargs):
        result = {}
        for key, typ in self.field_types.items():
            result[key] = typ.dictify(value[key])
        if self.extra_field_policy == 'save':
            for key, val in self._extras.items():
                if key not in result:
                    result[key] = val
        return result

    def validate(self, value, **kwargs):
        error_agg = ErrorAggregator(autoraise = kwargs.get('fail_early', False))
        with error_agg.checking():
            super(Mapping, self).validate(value, **kwargs)
        for field, type in self.field_types.iteritems():
            with error_agg.checking_sub(field):
                type.validate(value[field])
        error_agg.raise_if_any()


if __name__ == '__main__':
    import doctest
    doctest.testmod()



from base import Dictifier, Invalid
from dictifiable import Dictifiable

class UnknownType(Exception):
    '''Raised when a Polymorph encounters an unknown type.'''
    pass

class Polymorph(Dictifier):
    '''A dictifier that selects between several dictifiables based on classes.

    The mapping argument is a dict whose keys are strings and whose values are
    either (class_list, Dictifier) pairs or Dictifiable classes.

    Polymorph can Dictify any object that is an instance of one of the
    Dictifiable classes or any class in one of the class_list entries; it will
    dictify it as a tuple of (name, value), where name identifies which
    Dictifier was used and value is the dictified object; on undictifiance, it
    undictifies the value using the Dictifier for that name.

    For example:

    >>> from basic import Number, Complex
    >>> from schema import SchemaObj
    >>> class Point(SchemaObj):
    ...     field_types = dict(x=Number(), y=Complex())
    ...
    >>> num_or_point = Polymorph({'point':Point, 'num':((int, float), Number())})
    >>> num_or_point.dictify(12)
    ('num', 12)
    >>> num_or_point.dictify(3.5)
    ('num', 3.5)
    >>> num_or_point.dictify(Point(x=1, y=3)) == ('point', {'x':1, 'y':3})
    True
    >>> num_or_point.undictify(('num', 42))
    42.0
    >>> p = num_or_point.undictify(('point', {'x':-1, 'y':'2+2j'}))
    >>> p.x
    -1.0
    >>> p.y
    (2+2j)

    It will validate only the types in its list:

    >>> num_or_point.validate(3)
    >>> num_or_point.validate(3.5)
    >>> num_or_point.validate(p)
    >>> num_or_point.validate('hi')
    Traceback (most recent call last):
    ...
    Invalid: type_error
    '''
    def __init__(self, mapping):
        super(Polymorph, self).__init__()
        self.mapping = {}
        self.reverse_mapping = {}
        for name, data in mapping.items():
            if isinstance(data, type) and issubclass(data, Dictifiable):
                classes, dfier = (data,), data.dfier()
            else:
                classes, dfier = data
            self.mapping[name] = dfier
            for cls in classes:
                self.reverse_mapping[cls] = (name, dfier)

    def get_dfier(self, value):
        '''Given an object, get the corresponding name and dictifier.'''
        for basetype in type(value).__mro__:
            if basetype in self.reverse_mapping:
                return self.reverse_mapping[basetype]
        raise UnknownType(type(value).__name__)

    def undictify(self, value, **kwargs):
        '''Convert a dictified value to an object and return it.'''
        if not isinstance(value, (list, tuple)):
            raise Invalid('type_error')
        if len(value) != 2:
            raise Invalid('bad_list')
        name, value = value
        return self.mapping[name].undictify(value, **kwargs)

    def dictify(self, value, **kwargs):
        name, dfier = self.get_dfier(value)
        return (name, dfier.dictify(value, **kwargs))

    def validate(self, value, **kwargs):
        try:
            name, dfier = self.get_dfier(value)
        except UnknownType:
            raise Invalid("type_error")
        dfier.validate(value, **kwargs)

if __name__ == '__main__':
    import doctest
    doctest.testmod()



'''
Some tools for aggregating error messages.

These solve the problem where you wish to perform a number of subtasks,
continuing if any of the fail, and you want to aggregate all the errors
generated while doing this.

One use case for this is in form validation with human-readable output. A
simple, pythonic validation function would iterate over the fields in the form,
and would raise an exception as soon as it found any field that wasn't valid.
But when the errors are displayed to the user, you will likely want to show ALL
the errors on the form, not just the first one found.

So instead, you want to validate each field of the form regardless of whether or
not the previous fields were valid, and collect a list of all the errors that
came up, so that you can show them all to the user.

If the form has three fields called 'foo', 'bar', and 'baz', then you would want
to generate an exception with three sub-entries - one for each of these fields -
plus a list of any errors that are not specific to one field, such as violating
a constraint involving two fields' values.

The ErrorAggregator class provides tools for doing this, and the NestedException
class defines an Exception subclass well-suited to storing these errors.

'''

from contextlib import contextmanager


class NestedException(Exception):
    '''Exception with sub-exceptions.

    A NestedException has two useful fields, own_errors and sub_errors.

    own_errors is a list of root errors raised during some operation.

    sub_errors is a mapping from strings to NestedExceptions, representing the
    exceptions generated by named sub-steps in the operation.
    '''
    def __init__(self):
        self.own_errors = []
        self.sub_errors = {}
        super(NestedException, self).__init__()

    def __nonzero__(self):
        return bool(self.own_errors) or bool(self.sub_errors)

    def __repr__(self):
        pieces = [self.own_errors, self.sub_errors]
        return ', '.join(sorted(repr(x) for x in pieces if x))

    def __getitem__(self, item):
        if isinstance(item, int):
            return self.own_errors[item]
        return self.sub_errors[item]

    def add_own(self, exc):
        if isinstance(exc, NestedException):
            self.merge(exc)
        else:
            self.own_errors.append(exc)

    def add_sub(self, key, exc):
        '''Add a sub-exception.'''
        self.sub_errors.setdefault(key, type(self)()).add_own(exc)

    def merge(self, other):
        '''Merge another NestedException into this one.'''
        for own in other.own_errors:
            self.add_own(own)
        for key, sub in other.sub_errors.items():
            self.add_sub(key, sub)

    def __str__(self):
        own = ', '.join(str(err) for err in self.own_errors)
        other = ', '.join('{0}: [{1}]'.format(key, err) for (key, err) in self.sub_errors.items())
        return ' '.join(x for x in [own, other] if x)


class ErrorAggregator(object):
    '''Helper class for trying several things and accumulating all errors.

    Basic usage:
    >>> eg = ErrorAggregator()
    >>> eg.own_error(Exception("something_wrong"))
    >>> with eg.checking_sub("x"):
    ...     raise Exception("x_failed")
    >>> eg.has_errors()
    True
    >>> eg.error.own_errors
    [Exception('something_wrong',)]
    >>> eg.error.sub_errors
    {'x': [Exception('x_failed',)]}

    If autoraise is True, then instead of aggregating it will raise an error as
    soon as one is added:

    >>> eg = ErrorAggregator(autoraise=True)
    >>> eg.own_error(Exception("something_wrong"))
    Traceback (most recent call last):
        ...
    NestedException: something_wrong
    '''
    error_type = NestedException
    catch_type = Exception

    def __init__(self, autoraise=False):
        self.error = self.error_type()
        self.autoraise = autoraise

    def own_error(self, err):
        '''Add a self-error.'''
        self.error.add_own(err)
        if self.autoraise:
            raise self.error

    def sub_error(self, key, err):
        '''Add a child-error.'''
        self.error.add_sub(key, err)
        if self.autoraise:
            raise self.error

    @contextmanager
    def checking(self):
        '''Context manager for try-excepting tasks.'''
        try:
            yield
        except self.catch_type, e:
            self.own_error(e)

    @contextmanager
    def checking_sub(self, key):
        '''Context manager for try-excepting subtasks.'''
        try:
            yield
        except self.catch_type, e:
            self.sub_error(key, e)

    def has_errors(self):
        '''Returns true if the aggregator holds any errors.'''
        return bool(self.error)

    def raise_if_any(self):
        '''If the aggregator has any exceptions, raise a NestedException'''
        if self.has_errors():
            raise self.error


if __name__ == '__main__':
    import doctest
    doctest.testmod()



import re

from base import Invalid, Validator

class InRange(Validator):
    '''Require non-empty values to be between two endpoints.

    The endpoints themselves are allowed.

    >>> from basic import Int
    >>> from base import Field
    >>> RangeInt = Field(Int(), [InRange(0, 5)])
    >>> RangeInt.validate(0)
    >>> RangeInt.validate(3)
    >>> RangeInt.validate(5)
    >>> RangeInt.validate(None)
    >>> RangeInt.validate(-1)
    Traceback (most recent call last):
        ...
    Invalid: too_low
    >>> RangeInt.validate(6)
    Traceback (most recent call last):
        ...
    Invalid: too_high

    Note that the types can be any type for which < and > are defined:

    >>> from datetypes import Date
    >>> from datetime import date
    >>> from base import Field
    >>> range = InRange(date(2012, 1, 1), date(2012, 12, 31))
    >>> DateIn2012 = Field(Date(), [range])
    >>> DateIn2012.validate(date(2012, 4, 13))
    >>> DateIn2012.validate(date(2013, 1, 1))
    Traceback (most recent call last):
        ...
    Invalid: too_high
    '''
    def __init__(self, low=None, high=None):
        self.low = low
        self.high = high

    def validate(self, value, **kwargs):
        if value is None: return
        if self.low is not None and value < self.low:
            raise Invalid("too_low")
        if self.high is not None and value > self.high:
            raise Invalid("too_high")

class OneOf(Validator):
    '''Require non-empty values to come from a fixed list of options.

    >>> from basic import String
    >>> from base import Field
    >>> Baggins = Field(String(), [OneOf(['Frodo', 'Bilbo'])])
    >>> Baggins.validate("Bilbo")
    >>> Baggins.validate("Frodo")
    >>> Baggins.validate("Meriadoc")
    Traceback (most recent call last):
        ...
    Invalid: invalid_choice
    '''
    def __init__(self, options):
        self.options = options

    def validate(self, value, **kwargs):
        if value is None: return
        if value not in self.options:
            raise Invalid("invalid_choice")

class MatchesRegex(Validator):
    '''Require non-empty values to match a regular expression.

    >>> from basic import String
    >>> from base import Field
    >>> IP = Field(String(), vdators=[MatchesRegex("\d{1,3}(\.\d{1,3}){3}")])
    >>> IP.validate("127.0.0.1")
    >>> IP.validate("This is not an IP address.")
    Traceback (most recent call last):
        ...
    Invalid: invalid_string
    >>> IP.validate("127.0.0.no")
    Traceback (most recent call last):
        ...
    Invalid: invalid_string
    '''
    def __init__(self, regex):
        self.regex = re.compile(regex)

    def validate(self, value, **kwargs):
        if value is None: return
        if not self.regex.match(value):
            raise Invalid("invalid_string")


if __name__ == '__main__':
    import doctest
    doctest.testmod()



from base import Invalid, ErrorAggregator, Field
from dictifiable import Dictifiable
from subclass import Subclassable

def overlay(d1, d2, strip_none=True):
    '''Return a copy of d1 with d2 overlaid.

    If strip_none is False, this is equivalent to `d1.copy().update(d2)`.

    If strip_none is True, then any keys of d2 that map to None will be deleted
    from d1.
    '''
    x = d1.copy();
    x.update(d2)
    if strip_none:
        for key, value in d2.iteritems():
            if value is None:
                del x[key]
    return x

class SchemaObj(Subclassable, Dictifiable):
    extra_field_policy = "ignore"
    field_types = dict()

    def __init__(self, **kwargs):
        for key, value in kwargs.items():
            setattr(self, key, value)

    @classmethod
    def __subclass__(cls, field_types={}, **kwargs):
        '''Subclass this object.

        field_types will overlay any field_types set on this class. If
        field_types includes a pair (key, None), then field_types[key] will be
        deleted.
        '''
        super(SchemaObj, cls).__subclass__(**kwargs)
        cls.field_types = overlay(cls.field_types, field_types)

    def _handle_fields(self, value, error_agg, kwargs):
        '''Undictify our fields. Override this to customize subfields.'''
        for key, type in self.field_types.iteritems():
            with error_agg.checking_sub(key):
                val = value.get(key, None)
                setattr(self, key, type.undictify(val, **kwargs))

    def _handle_extras(self, value, error_agg, kwargs):
        '''Process any keys in value that aren't expected fields.

        If self.extra_field_policy is 'error', these extra values will be added
        as errors. Otherwise they'll be discarded.
        '''
        if self.extra_field_policy != 'error':
            return
        extra_keys = set(value.keys()) - set(self.field_types.keys())
        if extra_keys:
            error_agg.own_error(Invalid('unexpected_fields', keys=extra_keys))

    def __undictify__(self, value, **kwargs):
        if value is None:
            value = {}
        if not isinstance(value, dict):
            raise Invalid('not_dict')
        # If fail_early is True, then gather all errors from this and its
        # children. Otherwise, just raise the first error we encounter.
        error_agg = ErrorAggregator(autoraise = kwargs.get('fail_early', False))
        self._handle_fields(value, error_agg, kwargs)
        self._handle_extras(value, error_agg, kwargs)
        error_agg.raise_if_any()

    def __dictify__(self, **kwargs):
        result = {}
        for key, typ in self.field_types.items():
            result[key] = typ.dictify(getattr(self, key))
        return result

    def __validate__(self, **kwargs):
        error_agg = ErrorAggregator(autoraise = kwargs.get('fail_early', False))
        for field, type in self.field_types.iteritems():
            with error_agg.checking_sub(field):
                try:
                    val = getattr(self, field)
                except AttributeError:
                    raise Invalid("missing_field", name=field)
                type.validate(val)
        error_agg.raise_if_any()

    def dictify(self, **kwargs):
        return self.dfier().dictify(self, **kwargs)

    @classmethod
    def undictify(cls, value, **kwargs):
        return cls.dfier().undictify(value, **kwargs)

    def validate(self, **kwargs):
        return self.dfier().validate(self, **kwargs)


class DeclSchemaObj(SchemaObj):
    '''SchemaObj that automatically adds all Field class members to field_types.

    This is useful if you want to use SchemaObjs as mixins:

    >>> from basic import Int, String
    >>> class Point(DeclSchemaObj):
    ...   x = Field(Int())
    ...   y = Field(Int())
    >>> Point.field_types['x'] is Point.x
    True
    >>> Point.field_types['y'] is Point.y
    True
    >>> Point(x=1, y=3).dictify() == dict(x=1, y=3)
    True
    >>> class Labeled(DeclSchemaObj):
    ...   label = Field(String())
    >>> Labeled(label='hi').dictify() == dict(label='hi')
    True
    >>> class LabeledPoint(Labeled, Point):
    ...   pass
    >>> lp = LabeledPoint.undictify(dict(x='1', y='3', label='foo'))
    >>> lp.x, lp.y, lp.label
    (1, 3, u'foo')

    '''
    @classmethod
    def __subclass__(cls, **kwargs):
        super(DeclSchemaObj, cls).__subclass__(**kwargs)
        for key in dir(cls):
            val = getattr(cls, key)
            if isinstance(val, Field):
                cls.field_types[key] = val

if __name__ == '__main__':
    import doctest
    doctest.testmod()



from base import Invalid
from basic import Int, String
from schema import SchemaObj

class Point(SchemaObj):
    field_types = {
        'x': Int(),
        'y': Int(),
        'label': String(),
    }
    def __init__(self, x, y, label='foo'):
        super(Point, self).__init__()
        self.x = x
        self.y = y
        self.label = label

plain_data = {'x':-12, 'y':1, 'label':'foo'}
blank_data = {}
bad_data = {'x':'hi', 'y':-1, 'label':'blarg'}
extra_data = {'x':-12, 'y':1, 'label':'foo', 'ex':['foo','bar'], 'ex2':'hi'}

def test_basic():
    p = Point.undictify(plain_data)
    assert p.x == -12
    assert p.y == 1
    assert p.label == 'foo'
    assert p.dictify() == plain_data, p.dictify()

def test_errors():
    try:
        Point.undictify(bad_data)
    except Invalid, e:
        assert e.sub_errors['x'][0].message == 'type_error'
    else:
        assert False

def test_ban_extra():
    StrictPoint = Point.subclass(extra_field_policy='error')
    try:
        StrictPoint.undictify(extra_data)
    except Invalid, e:
        assert len(e.own_errors) == 1
        err = e.own_errors[0]
        assert err.message == 'unexpected_fields'
        assert err.kwargs['keys'] == {'ex', 'ex2'}
    else:
        assert False

def test_construct():
    p = Point(-12,1)
    assert p.dictify() == plain_data


class TwoPoints(SchemaObj):
    field_types = dict(a=Point, b=Point)

def test_nested():
    x = TwoPoints.undictify(dict(a=plain_data, b=plain_data))
    assert x.a.x == x.b.x == -12
    assert x.dictify() == dict(a=plain_data, b=plain_data)



from base import Invalid, Field
from basic import Int, String
from schema import SchemaObj

from common_validators import InRange, OneOf, MatchesRegex

class Point(SchemaObj):
    field_types = {
        'x': Field(Int(), [InRange(0,10)]),
        'y': Field(Int(), [OneOf([1,2,3,4,5])]),
        'label': Field(String(), [MatchesRegex("foo[0-9]*")]),
    }

valid_str = {'x':'5', 'y':'3', 'label':'foo12'}
invalid_str = {'x':'-1', 'y':'0', 'label':'bar1'}
valid_data = {'x':5, 'y':3, 'label':'foo12'}
invalid_data = {'x':-1, 'y':0, 'label':'bar1'}

def test_valid():
    p = Point.undictify(valid_str)
    assert p.x == 5
    assert p.y == 3
    assert p.label == 'foo12'
    p.validate()
    assert p.dictify() == valid_data

def test_invalid():
    p = Point.undictify(invalid_str)
    assert p.x == -1
    assert p.y == 0
    assert p.label == 'bar1'
    assert p.dictify() == invalid_data
    expected_errors = dict(
        x = 'too_low',
        y = 'invalid_choice',
        label = 'invalid_string')
    try:
        p.validate(fail_early=True)
    except Invalid, e:
        assert not e.own_errors
        assert len(e.sub_errors) == 1, e
        k = e.sub_errors.keys()[0]
        assert e.sub_errors[k][0].message == expected_errors[k], e.sub_errors[k][0].message +' vs. '+ expected_errors[k]
    else:
        assert False, "Should have raised NestedInvalid!"
    try:
        p.validate()
    except Invalid, e:
        assert not e.own_errors
        assert len(e.sub_errors) == 3
        for k in expected_errors:
            assert e.sub_errors[k][0].message == expected_errors[k]
    else:
        assert False, "Should have raised NestedInvalid!"



from base import Dictifier, ErrorAggregator, Invalid, Field

class List(Dictifier):
    '''Dictifier for homogenous lists

    The argument elt_type is the dictifier for the elements in the list; if it
    is not a Field, it will be wrapped in one.

    The arugment required determines the generated Field's not_empty flag. It is
    ignored if elt_type is a subclass of Field.

    For example:

    >>> from basic import Int
    >>> l = List(Int())
    >>> l.undictify(["1", "4", 5])
    [1, 4, 5]
    >>> l.dictify([1, 2, 3])
    [1, 2, 3]
    >>> l.validate([1, 2, 3])

    By default empty values are not allowed:

    >>> l.validate([1, None, 'hi'])
    Traceback (most recent call last):
    ...
    Invalid: 1: [empty], 2: [type_error]

    These two are equivalent:

    >>> l1 = List(Field(Int(), not_empty=False))
    >>> l2 = List(Int(), required=False)
    >>> l1.validate([1, None, 3])
    >>> l2.validate([1, None, 3])
    '''
    def __init__(self, elt_type, required=True):
        if not isinstance(elt_type, Field):
            elt_type = Field(elt_type, not_empty=required)
        self.elt_type = elt_type

    def undictify(self, value, **kwargs):
        # If _full_errors is True, then gather all errors from this and its
        # children. Otherwise, just raise the first error we encounter.
        error_agg = ErrorAggregator(autoraise = kwargs.get('fail_early', False))
        result = []
        for i, item in enumerate(value):
            with error_agg.checking_sub(str(i)):
                result.append(self.elt_type.undictify(item, **kwargs))
        error_agg.raise_if_any()
        return result

    def dictify(self, value, **kwargs):
        return [self.elt_type.dictify(x, **kwargs) for x in value]

    def validate(self, value, **kwargs):
        error_agg = ErrorAggregator(autoraise = kwargs.get('fail_early', False))
        try:
            iterator = iter(value)
        except TypeError: # item isn't iterable!
            raise Invalid("type_error")
        for i, item in enumerate(iterator):
            with error_agg.checking_sub(str(i)):
                self.elt_type.validate(item)
        error_agg.raise_if_any()


if __name__ == '__main__':
    import doctest
    doctest.testmod()



from base import Invalid
from mapping import Mapping

class PyType(Mapping):
    '''Dictifier for python objects with constructor functions.

    This is a Mapping-based dictifier for objects that are described by a small
    number of attributes.

    The field_types arg specifies the types of the attributes of the objects you
    want to be able to dictify/undictify. These fields must all be exposed attrs
    of any instance you wish to dictify.

    The cls arg specifies the type of object you want to create.

    The ctor arg specifies a function that takes as keyword arguments the values
    named by the field_types argument. If ctor isn't provided, it defaults to
    the cls constructor.

    For example, suppose you're using the MailMessage class from above:

    >>> class MailMessage(object):
    ...     def __init__(self, subject, sender, body=None):
    ...         self.body, self.subject, self.sender = body, subject, sender

    You could build a dictifier for it much like

    >>> from basic import String
    >>> msg_dfier = PyType(
    ...     field_types = dict(
    ...         body = String(),
    ...         sender = String(),
    ...         subject = String()),
    ...     cls = MailMessage)

    And now msg_dfier can dictify/undictify MailMessages:

    >>> m = MailMessage("Business", "sender@foo.net")
    >>> data = msg_dfier.dictify(m)
    >>> sorted(data.items())
    [('body', None), ('sender', 'sender@foo.net'), ('subject', 'Business')]
    >>> m2 = msg_dfier.undictify(data)
    >>> isinstance(m2, MailMessage)
    True
    >>> (m2.body, m2.subject, m2.sender) == (m.body, m.subject, m.sender)
    True
    >>> msg_dfier.validate(m2)
    >>> msg_dfier.validate("not a message")
    Traceback (most recent call last):
    ...
    Invalid: type_error
    '''
    def __init__(self, field_types, cls, ctor=None, extra_field_policy='ignore'):
        super(PyType, self).__init__(field_types, extra_field_policy)
        self.cls = cls
        self.ctor = ctor or cls

    def undictify(self, value, **kwargs):
        '''Use the Schema machinery, then extract to a new object.'''
        mapping = super(PyType, self).undictify(value, **kwargs)
        return self.ctor(**mapping)

    def _obj_to_mapping(self, obj):
        mapping = {}
        for key in self.field_types:
            mapping[key] = getattr(obj, key)
        return mapping

    def dictify(self, value, **kwargs):
        mapping = self._obj_to_mapping(value)
        return super(PyType, self).dictify(mapping, **kwargs)

    def validate(self, value, **kwargs):
        if not isinstance(value, self.cls):
            raise Invalid('type_error')
        mapping = self._obj_to_mapping(value)
        super(PyType, self).validate(mapping, **kwargs)


if __name__ == '__main__':
    import doctest
    doctest.testmod()



'''subclass.py: metaclass for programmatically subclassable classes.

Subclasses of the Subclassable class have classmethods called subclass(**kwargs)
and __subclass__(**kwargs).

Whenever such a class is subclassed, the __subclass__() method is called and
passed the contents of the new class's dictionary.

cls.subclass(**kwargs) is equivalent to creating a new subclass with the
contents of kwargs written out within it.

For example:

>>> class Point(Subclassable):
...   round = False
...   def __init__(self, x, y):
...     if self.round:
...        x, y = int(x), int(y)
...     self.x, self.y = x, y
...
>>> p = Point(.5, 3.25)
>>> p.x, p.y
(0.5, 3.25)

The following two subclasses are (nearly) equivalent

>>> class IntPoint1(Point):
...   round = True
...
>>> IntPoint2 = Point.subclass(round=True)
>>> p1 = IntPoint1(.5, 3.25)
>>> p2 = IntPoint2(.5, 3.25)
>>> p1.x, p1.y
(0, 3)
>>> p2.x, p2.y
(0, 3)

The only difference is in their names:

>>> IntPoint1.__name__
'IntPoint1'
>>> IntPoint2.__name__
'Point_sub'

This can be rectified via the special __class_name attribute:

>>> IntPoint3 = Point.subclass(round=True, __class_name="IntPoint1")
>>> IntPoint3.__name__
'IntPoint1'

'''


class SubclassableMeta(type):
    '''Metaclass for classes that can easily programmatically subclass.'''
    def __new__(cls, name, supers, kwargs):
        t = type.__new__(cls, name, supers, {})
        # Force __subclass__ to be a classmethod
        if not isinstance(t.__subclass__, classmethod):
            t.__subclass__ = classmethod(t.__subclass__.im_func)
        t.__subclass__(**kwargs)
        return t

    def __subclass__(cls, **kwargs):
        for name, val in kwargs.items():
            setattr(cls, name, val)

    def subclass(cls, **kwargs):
        '''
        Programmatically generate subclasses of this class.

        The generated subclass will be called <name of this class>_sub unless
        the special parameter __class_name is passed in.
        '''
        if '__class_name' in kwargs:
            class_name = kwargs.pop('__class_name')
        else:
            class_name = cls.__name__ + "_sub"
        subcls = type(class_name, (cls,), kwargs)
        return subcls



class Subclassable(object):
    '''Base class for programmatically subclassable types.'''
    __metaclass__ = SubclassableMeta

if __name__ == '__main__':
    import doctest
    doctest.testmod()



#!/usr/bin/env python

from distutils.core import setup

setup(
    name='dfiance',
    version='0.1.1',
    license='BSD',
    author="Daniel Lepage",
    author_email="dplepage@gmail.com",
    packages=['dfiance',],
    long_description=open('README.txt').read(),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python :: 2",
    ]
)


from base import Dictifier, Invalid

class _EmptyClass:
    '''Placeholder class for undictifying Dictifiable instances.'''
    pass


class _EmptyNSClass(object):
    '''Placeholder class for undictifying Dictifiable instances.'''
    pass


def empty_instance(cls):
    '''Create an instance of cls without invoking cls.__init__'''
    if issubclass(cls, object):
        instance = _EmptyNSClass()
    else:
        instance = _EmptyClass()
    instance.__class__ = cls
    return instance


class DictifiableDictifier(Dictifier):
    '''Dictifier for Dictifiables.

    In general users will not create instances of this class directly, but will
    instead use the classmethod Dictifiable.dfier(), which returns an instance
    of this class for that Dictifiable type.
    '''
    def __init__(self, dictifiable_cls):
        self.cls = dictifiable_cls

    def dictify(self, value, **kwargs):
        return value.__dictify__(**kwargs)

    def undictify(self, value, **kwargs):
        instance = empty_instance(self.cls)
        instance.__undictify__(value, **kwargs)
        return instance

    def validate(self, value, **kwargs):
        if not isinstance(value, self.cls):
            raise Invalid("type_error", expected=self.cls.__name__)
        value.__validate__(**kwargs)


class Dictifiable(object):
    '''An object that knows how to dictify/undictify itself.

    The class should implement three functions:

    __dictify__(self, **kwargs): dictify self
    __undictify__(self, value, **kwargs): undictify self from value
    __validate__(self, **kwargs): validate self

    __dictify__ and __validate__ are straightforward; __undictify__ will be
    called on a blank instance of the class (i.e., without calling __init__), so
    it must set up the new instance from scratch.

    The classmethod .dfier() returns a Dictifier for this class.

    Dictifiable classes also expose three helper methods:

    self.dictify(**kwargs) - same as cls.dfier().dictify(self, **kwargs)
    cls.undictify(val, **kwargs) - same as cls.dfier().undictify(val, **kwargs)
    self.validate(**kwargs) - same as cls.dfier().validate(self, **kwargs)

    Note that as a consequence, the class itself very nearly adheres to the
    dictify protocol, since the instance methods dictify and validate can be
    called as classmethods and passed instances as arguments. However,
    cls.validate will break if the value is not an instance of the class,
    because python doesn't allow classmethods to be called on non-instances.

    Thus, you should use cls.dfier() if you need the dictifier for this class,
    instead of just using the class.

    '''
    # This class is almost itself a dictifier, but it isn't quite, because the
    # class's validate functions will crash given a value that isn't an instance
    # Python 3 will fix this, because instance methods are callable as
    # classmethods with any argument for self
    @classmethod
    def dfier(cls):
        return DictifiableDictifier(cls)

    def __dictify__(self, **kwargs):
        raise NotImplementedError()

    def __undictify__(self, value, **kwargs):
        raise NotImplementedError

    def __validate__(self, **kwargs):
        pass

    def dictify(self, **kwargs):
        return self.dfier().dictify(self, **kwargs)

    @classmethod
    def undictify(cls, value, **kwargs):
        return cls.dfier().undictify(value, **kwargs)

    def validate(self, **kwargs):
        return self.dfier().validate(self, **kwargs)


if __name__ == '__main__':
    import doctest
    doctest.testmod()



from error_aggregator import ErrorAggregator as _ErrAgg, NestedException


class SingleInvalid(Exception):
    '''Base class for all undictification errors.

    In general, self.message will be a string indicating the specific error,
    e.g. 'required' when a value is missing that shouldn't be, 'not_int' for
    a value that must be an integer but cannot be parsed as one, and so forth.
    The specific errors raised by various Dictifiers are class-specific.
    '''
    def __init__(self, message=None, **kwargs):
        if kwargs:
            super(SingleInvalid, self).__init__(message, kwargs)
        else:
            super(SingleInvalid, self).__init__(message)
        self.message = message
        self.kwargs = kwargs


class Invalid(NestedException):
    '''Base class for undictification errors of nested objects.

    This allows dictifiable types with sub-fields to record the errors from
    their subfields as well.
    '''
    def __init__(self, message=None, **kwargs):
        super(Invalid, self).__init__()
        if message or kwargs:
            self.own_errors.append(SingleInvalid(message, **kwargs))


class ErrorAggregator(_ErrAgg):
    '''Specialized ErrorAggregator to only aggregate Invalids'''
    error_type = Invalid
    catch_type = Invalid


class Validator(object):
    '''Abstract base class for Validators.'''
    def validate(self, value, **kwargs):
        return


class Dictifier(Validator):
    '''Abstract base class for Dictifiers.'''
    def dictify(self, value, **kwargs):
        raise NotImplementedError()

    def undictify(self, value, **kwargs):
        raise NotImplementedError()


class Field(Dictifier):
    '''Wrapper around a dictifier and an optional list of validators.

    A Field behaves like the dictifier handed to it on intialization, except in
    two respects:

    1. It handles None values
    2. It can have additional validators attached to it

    It passes None through dictify unchanged; on validation or undictifiance, it
    treats None as valid unless self.not_empty is True, in which case it raises
    Invalid("empty")

    If a list of validators is passed on initialization, then field.validate()
    will call each validator in order and raise a NestedInvalid with all of
    their exceptions if any of them fail.
    '''
    def __init__(self, dfier, vdators=(), not_empty=False):
        self.dfier = dfier
        self.vdators = tuple(vdators)
        self.not_empty = not_empty

    def dictify(self, value, **kwargs):
        if value is None:
            return None
        return self.dfier.dictify(value, **kwargs)

    def undictify(self, value, **kwargs):
        if value is None:
            if self.not_empty:
                raise Invalid("empty")
            return None
        return self.dfier.undictify(value, **kwargs)

    def validate(self, value, **kwargs):
        if value is None:
            if self.not_empty:
                raise Invalid("empty")
            return
        fail_early = kwargs.get("dfy_fail_early", False)
        error_agg = ErrorAggregator(autoraise = fail_early)
        with error_agg.checking():
            self.dfier.validate(value)
        for vdator in self.vdators:
            with error_agg.checking():
                vdator.validate(value)
        error_agg.raise_if_any()

    @classmethod
    def asfield(cls, dfier):
        '''Wrap a dictifier in a Field if it isn't already one.'''
        if isinstance(dfier, Field):
            return dfier
        return Field(dfier)

def _doctest_field_tests():
    '''
    Empty function so I can write tests in doctest form.

    >>> from basic import Int
    >>> field = Field(Int())
    >>> field.dictify(None)
    >>> field.undictify(None)
    >>> field.validate(None)
    >>> req_field = Field(Int(), not_empty=True)
    >>> req_field.dictify(None)
    >>> req_field.undictify(None)
    Traceback (most recent call last):
    ...
    Invalid: empty
    >>> req_field.validate(None)
    Traceback (most recent call last):
    ...
    Invalid: empty
    '''


if __name__ == '__main__':
    import doctest
    doctest.testmod()



from subclass import Subclassable
from base import Invalid, ErrorAggregator, Field, Dictifier, Validator
from basic import String, Boolean, Int, Number, Complex, TypeDictifier
from datetypes import DateTime, Date, Time
from list import List
from mapping import Mapping
from pytype import PyType
from dictifiable import Dictifiable
from schema import SchemaObj
from polymorph import Polymorph
import common_validators as validators

__all__ = ["Subclassable", "Invalid", "ErrorAggregator", "Field", "Dictifier",
           "Validator", "String", "Boolean", "Int", "Number", "Complex",
           "TypeDictifier", "DateTime", "Date", "Time", "List", "Mapping",
           "PyType", "Dictifiable", "SchemaObj", "Polymorph", "validators"]



import numbers

from base import Dictifier, Invalid

class TypeDictifier(Dictifier):
    '''Dictifier for simple types, such as integers and strings.

    Dfiance is a no-op:

    >>> intdfy = TypeDictifier(int)
    >>> intdfy.dictify(4)
    4

    Undfiance calls self.type:

    >>> intdfy.undictify(4)
    4
    >>> intdfy.undictify("4")
    4
    >>> intdfy.undictify("hello")
    Traceback (most recent call last):
    ...
    Invalid: type_error

    Validation uses isinstance:

    >>> intdfy.validate(3)
    >>> intdfy.validate(3.5)
    Traceback (most recent call last):
    ...
    Invalid: type_error
    '''
    type = None
    error_msg = 'type_error'
    allowed_types = None

    def __init__(self, type=None, error_msg=None, allowed_types=None):
        if type:
            self.type = type
        if error_msg:
            self.error_msg = error_msg
        if allowed_types:
            self.allowed_types = tuple(allowed_types)
        if not self.allowed_types:
            self.allowed_types = (self.type,)

    def dictify(self, value, **kwargs):
        return value

    def undictify(self, value, **kwargs):
        try:
            return self.type(value)
        except (ValueError, TypeError):
            raise Invalid(self.error_msg)

    def validate(self, value, **kwargs):
        if not isinstance(value, self.allowed_types):
            raise Invalid(self.error_msg)


class String(TypeDictifier):
    '''
    >>> s = String()
    >>> s.undictify("foo")
    u'foo'
    >>> s.dictify("foo")
    'foo'
    >>> s.dictify(u"foo")
    u'foo'
    '''
    type = unicode
    # Undictify always returns unicode, but any string is valid.
    allowed_types = basestring

class Boolean(TypeDictifier):
    r"""
    >>> b = Boolean()
    >>> b.undictify(True)
    True
    >>> b.undictify('')
    False
    >>> # Note that this doesn't parse strings:
    >>> b.undictify('false')
    True
    >>> b.dictify(False)
    False
    >>> b.validate(True)
    >>> b.validate(False)
    >>> b.validate(1)
    Traceback (most recent call last):
    ...
    Invalid: type_error
    """
    type = bool

class Int(TypeDictifier):
    r"""
    >>> i = Int()
    >>> i.undictify('10')
    10
    >>> i.dictify(i.undictify('10'))
    10
    >>> i.undictify('10.0')
    Traceback (most recent call last):
    ...
    Invalid: type_error
    >>> i.undictify('one')
    Traceback (most recent call last):
    ...
    Invalid: type_error
    """
    type = int
    allowed_types = numbers.Integral

class Number(TypeDictifier):
    r"""
    >>> num = Number()
    >>> num.undictify('10')
    10.0
    >>> num.dictify(num.undictify('10'))
    10.0
    >>> num.undictify('10.1')
    10.1
    >>> num.dictify(num.undictify('10.1'))
    10.1
    >>> num.validate(1)
    >>> num.validate(1.1)
    >>> num.validate(1+1j)
    Traceback (most recent call last):
    ...
    Invalid: type_error
    >>> num.undictify('ten')
    Traceback (most recent call last):
    ...
    Invalid: type_error
    """
    type = float
    allowed_types = numbers.Real

class Complex(TypeDictifier):
    r"""
    >>> com = Complex()
    >>> com.undictify('10')
    (10+0j)
    >>> com.dictify(com.undictify('10+j'))
    (10+1j)
    >>> com.undictify('10.1-2.3j')
    (10.1-2.3j)
    >>> com.dictify(com.undictify('10.1'))
    (10.1+0j)
    >>> com.validate(1)
    >>> com.validate(1.1)
    >>> com.validate(1+1j)
    >>> com.undictify('ten')
    Traceback (most recent call last):
    ...
    Invalid: type_error
    """
    type = complex
    allowed_types = numbers.Complex

if __name__ == '__main__':
    import doctest
    doctest.testmod()


import datetime

try:
    from dateutil.parser import parse as parse_datetime
    _have_dateutil = True
except ImportError:
    _have_dateutil = False

from base import Invalid, Dictifier

class DateTime(Dictifier):
    """
    Dictifier for datetimes.

    The 'format' argument determines what format will be used for serializing,
    and if the 'require_format' argument is True, only strings in that format
    will be deserialized. 'require_format' must be true if the dateutil module
    is not installed.

    If you have dateutil installed, however, then 'require_format' defaults to
    False, and DateTime will use dateutil.parser.parse to process dates, so any
    string dateutil can parse can be undictified:

    >>> dtfier = DateTime()
    >>> dtfier.undictify("12/25/2003")
    datetime.datetime(2003, 12, 25, 0, 0)
    >>> dtfier.undictify("12/25/2003 14:25:03.1")
    datetime.datetime(2003, 12, 25, 14, 25, 3, 100000)
    >>> dtfier.undictify("Dec 25, 2003 2:25:03.1 pm")
    datetime.datetime(2003, 12, 25, 14, 25, 3, 100000)


    Invalid strings get specific error messages:

    >>> dtfier.undictify("12/25/20003")
    Traceback (most recent call last):
    ...
    Invalid: bad_year
    >>> dtfier.undictify("13/25/2003")
    Traceback (most recent call last):
    ...
    Invalid: bad_month
    >>> dtfier.undictify("12/32/2003")
    Traceback (most recent call last):
    ...
    Invalid: bad_day
    >>> dtfier.undictify("12/25/2003 26:25:03.1")
    Traceback (most recent call last):
    ...
    Invalid: bad_hour
    >>> dtfier.undictify("12/25/2003 14:71:03.1")
    Traceback (most recent call last):
    ...
    Invalid: bad_minute
    >>> dtfier.undictify("12/25/2003 14:25:93.1")
    Traceback (most recent call last):
    ...
    Invalid: bad_second


    Non-numbers get generic "bad_format" error messages:

    >>> dtfier.undictify("12/hello/2003")
    Traceback (most recent call last):
    ...
    Invalid: bad_format
    >>> dtfier.undictify("Not even remotely a datetime.")
    Traceback (most recent call last):
    ...
    Invalid: bad_format
    >>> dtfier.undictify(["this", "isn't", "even", "a", "string"])
    Traceback (most recent call last):
    ...
    Invalid: type_error


    Dictifying will serialize to self.format, which defaults to
    "%m/%d/%Y %H:%M:%S.%f" but can be set to anything.

    >>> d = datetime.datetime(2003, 12, 25, 14, 21, 3, 100500)
    >>> DateTime().dictify(d)
    u'12/25/2003 14:21:03.100500'
    >>> DateTime(format='%I:%M:%S.%f%p on %h %e, %Y').dictify(d)
    u'02:21:03.100500PM on Dec 25, 2003'


    Be warned, though, that you can fail at formatting by entering something
    which neither datetime.strptime nor dateutil can parse:

    >>> dumb_format = 'At %I:%M:%S.%f%p on day number %e of the year %Y, in %h'
    >>> dtfier = DateTime(format=dumb_format)
    >>> dtfier.dictify(d)
    u'At 02:21:03.100500PM on day number 25 of the year 2003, in Dec'
    >>> dtfier.undictify(dtfier.dictify(d))
    Traceback (most recent call last):
    ...
    Invalid: bad_format


    Or by entering something that throws away data:

    >>> dtfier = DateTime(format="%h %e %Y")
    >>> dtfier.dictify(d)
    u'Dec 25 2003'
    >>> dtfier.undictify(dtfier.dictify(d))
    datetime.datetime(2003, 12, 25, 0, 0)
    >>> dtfier.undictify(dtfier.dictify(d)) == d
    False


    Validate only approves of datetimes:

    >>> DateTime().validate(datetime.datetime.now())
    >>> DateTime().validate("")
    Traceback (most recent call last):
    ...
    Invalid: type_error


    Like all good dictifiers, undictify is idempotent:

    >>> dtfier = DateTime()
    >>> dt = dtfier.undictify("1/1/2003")
    >>> dtfier.undictify(dt) == dt
    True
    """
    _dt_type = datetime.datetime

    def __init__(self, format="%m/%d/%Y %H:%M:%S.%f", require_format=not _have_dateutil):
        self.format = format
        if not require_format and not _have_dateutil:
            msg = ("Can't parse dates or times with require_format=False"
                   " unless dateutil is installed.")
            raise ValueError(msg)

    def parse_datetime(self, value):
        '''Parse a formatted datetime.

        Tries datetime.datetime.strptime; if that fails it falls back to
        dateutil if present.
        '''
        if not isinstance(value, basestring):
            raise Invalid("type_error")
        try:
            return datetime.datetime.strptime(value, self.format)
        except ValueError, e:
            if not _have_dateutil:
                raise Invalid("bad_format")
        # If we get here, then we have dateutil - try to use it!
        try:
            return parse_datetime(value)
        except ValueError, e:
            if e.message.lower() == 'year is out of range':
                raise Invalid("bad_year")
            elif e.message.lower() == 'month must be in 1..12':
                raise Invalid("bad_month")
            elif e.message.lower() == 'day is out of range for month':
                raise Invalid("bad_day")
            elif e.message.lower() == 'hour must be in 0..23':
                raise Invalid('bad_hour')
            elif e.message.lower() == 'minute must be in 0..59':
                raise Invalid('bad_minute')
            elif e.message.lower() == 'second must be in 0..59':
                raise Invalid('bad_second')
            else:
                raise Invalid("bad_format")

    def undictify(self, value, **kwargs):
        if isinstance(value, self._dt_type):
            return value
        return self.parse_datetime(value)

    def validate(self, value):
        if not isinstance(value, self._dt_type):
            raise Invalid("type_error")

    def dictify(self, value):
        return unicode(value.strftime(self.format))

class Date(DateTime):
    """
    Dictifier for dates.

    All the notes about DateTime apply:

    Dateutil parsing:

    >>> date = Date()
    >>> date.undictify("12/25/2003")
    datetime.date(2003, 12, 25)

    Error msgs for each field that can fail, contingent on dateutil recognizing
    that something is the field in question:

    >>> date.undictify("13/25/2003")
    Traceback (most recent call last):
    ...
    Invalid: bad_month
    >>> date.undictify("12/32/2003")
    Traceback (most recent call last):
    ...
    Invalid: bad_day
    >>> date.undictify("12/25/20003")
    Traceback (most recent call last):
    ...
    Invalid: bad_year
    >>> date.undictify("12/hi/2003")
    Traceback (most recent call last):
    ...
    Invalid: bad_format
    >>> date.undictify("Not a date")
    Traceback (most recent call last):
    ...
    Invalid: bad_format

    Formats work, but don't choose something dateutil can't read back:

    >>> date = datetime.date(2003, 12, 25)
    >>> datefier = Date(format="%h %e '%g")
    >>> datefier.dictify(date)
    u"Dec 25 '03"
    >>> datefier.undictify(datefier.dictify(date))
    datetime.date(2003, 12, 25)
    >>> datefier = Date(format="Day %e of month %h in the year %g")
    >>> datefier.dictify(date)
    u'Day 25 of month Dec in the year 03'
    >>> datefier.undictify(datefier.dictify(date))
    Traceback (most recent call last):
    ...
    Invalid: bad_format

    Validate passes only dates:

    >>> Date().validate(datetime.date.today())
    >>> Date().validate(datetime.time())
    Traceback (most recent call last):
    ...
    Invalid: type_error

    As a side effect of dateutil, the Date dictifier WILL undictify strings
    dateutil parses as full datetimes, discarding the time:

    >>> Date().undictify('12/25/2003 14:21:03.100500')
    datetime.date(2003, 12, 25)
    """

    _dt_type = datetime.date

    def __init__(self, format="%m/%d/%Y"):
        super(Date, self).__init__(format)

    def undictify(self, value, **kwargs):
        if isinstance(value, self._dt_type):
            return value
        return self.parse_datetime(value).date()

class Time(DateTime):
    """
    >>> Time().undictify("14:25:03.1")
    datetime.time(14, 25, 3, 100000)
    >>> Time().undictify("26:25:03.1")
    Traceback (most recent call last):
    ...
    Invalid: bad_hour
    >>> Time().undictify("14:71:03.1")
    Traceback (most recent call last):
    ...
    Invalid: bad_minute
    >>> Time().undictify("14:25:93.1")
    Traceback (most recent call last):
    ...
    Invalid: bad_second
    >>> Time().undictify("Not a time")
    Traceback (most recent call last):
    ...
    Invalid: bad_format
    >>> Time().dictify(datetime.time(14, 25, 3, 100000))
    u'14:25:03.100000'
    >>> Time(format="%I:%M%p").dictify(datetime.time(14, 25, 3, 1))
    u'02:25PM'
    >>> Time().undictify(datetime.time(14, 25, 3, 1))
    datetime.time(14, 25, 3, 1)

    """
    _dt_type = datetime.time

    def __init__(self, format="%H:%M:%S.%f"):
        super(Time, self).__init__(format)

    def undictify(self, value, **kwargs):
        if isinstance(value, self._dt_type):
            return value
        return self.parse_datetime(value).time()

if __name__ == '__main__':
    import doctest
    doctest.testmod()